@@ -128,6 +128,18 @@ class Agent(object):
         self.keep_awake = False  # this prevents the agent from entering sleep mode when there
         # are no new messages.
 
+    def __str__(self):
+        return f"{self.__class__.__name__}({self.uuid})"
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}({self.uuid})"
+
+    @property
+    def time(self):
+        """ returns time as float"""
+        assert isinstance(self._scheduler_api, Scheduler), "agent must be added to scheduler using scheduler.add(agent)"
+        return self._scheduler_api.clock.time
+
     @property
     def uuid(self):
         """
@@ -143,11 +155,15 @@ class Agent(object):
     def uuid(self, value):
         raise ValueError("UUID cannot be set once the object has been instantiated")
 
-    def __str__(self):
-        return f"{self.__class__.__name__}({self.uuid})"
-
-    def __repr__(self):
-        return f"{self.__class__.__name__}({self.uuid})"
+    @property
+    def messages(self):
+        """
+        :return: Boolean: True if there are messages.
+        """
+        if self.inbox:
+            return True
+        else:
+            return False
 
     def send(self, msg):
         """ The only method for sending messages in the system.
@@ -160,16 +176,6 @@ class Agent(object):
         assert isinstance(self._scheduler_api, Scheduler), "agent must be added to scheduler using scheduler.add(agent)"
         self._scheduler_api.mail_queue.append(msg)
 
-    @property
-    def messages(self):
-        """
-        :return: Boolean: True if there are messages.
-        """
-        if self.inbox:
-            return True
-        else:
-            return False
-
     def receive(self):
         """
         :return: Returns AgentMessage if any.
@@ -291,6 +297,16 @@ class Agent(object):
                                       relative=relative,
                                       ignore_alarm_if_idle=ignore_alarm_if_idle)
 
+    def list_alarms(self, receiver=None):
+        """ returns list of alarms set by agent.
+        :param: receiver - optional if None receiver is self.uuid
+        :return: tuple (clock time, alarm message)
+        """
+        if receiver is None:
+            receiver = self.uuid
+        assert isinstance(self._scheduler_api, Scheduler)
+        return self._scheduler_api.list_alarms(receiver)
+
     def remove_alarm(self, alarm_message=None):
         """
         :param alarm_message: optional, if provided, the alarm associated with the given message is removed.
@@ -300,9 +316,9 @@ class Agent(object):
         """
         self._scheduler_api.clear_alarms(uuid=self.uuid, alarm_message=alarm_message)
 
-    def subscribe(self, uuid=None, topic=None):
+    def subscribe(self, target=None, topic=None):
         """
-        :param uuid: optional, the uuid of the agent that self wants to subscribe to.
+        :param target: optional, the uuid of the agent that self wants to subscribe to.
         :param topic: optional, the topic of the message that self want to subscribe to.
 
         A method to be used by the agent to set and subscribe to a particular topic
@@ -318,7 +334,7 @@ class Agent(object):
 
         """
         assert isinstance(self._scheduler_api, Scheduler), "agent must be added to scheduler using scheduler.add(agent)"
-        self._scheduler_api.subscribe(subscriber=self.uuid, target=uuid, topic=topic)
+        self._scheduler_api.subscribe(subscriber=self.uuid, target=target, topic=topic)
 
     def unsubscribe(self, target=None, topic=None):
         """ A method to be used by the agent to unset and unsubscribe to a particular topic
@@ -349,10 +365,10 @@ class Agent(object):
         assert isinstance(self._scheduler_api, Scheduler), "agent must be added to scheduler using scheduler.add(agent)"
         self._scheduler_api.add(agent)
 
-    def remove(self, uuid):
+    def remove(self, agent):
         """ Removes the agent from the scheduler. """
         assert isinstance(self._scheduler_api, Scheduler), "agent must be added to scheduler using scheduler.add(agent)"
-        self._scheduler_api.remove(uuid)
+        self._scheduler_api.remove(agent)
 
 
 class SchedulerException(MasLiteException):
@@ -384,13 +400,12 @@ class Clock(object):
         for timestamp in self.alarms:  # alarms are already sorted.
             if timestamp > self._time:
                 return
-            for message in self.alarm_messages[timestamp]:
-                self.scheduler_api.mail_queue.append(message)
+            self.scheduler_api.mail_queue.extend(self.alarm_messages[timestamp][:])
             del self.alarm_messages[timestamp]
             self.alarms.remove(timestamp)
 
     def set_alarm(self, delay, signal, ignore_alarm_if_idle):
-        assert isinstance(delay, (int,float))
+        assert isinstance(delay, (int, float))
         assert isinstance(signal, AgentMessage)
         assert isinstance(ignore_alarm_if_idle, bool)
         wakeup_time = self.time + delay
@@ -399,17 +414,29 @@ class Clock(object):
         insort(self.alarms, wakeup_time)  # smallest first!
         self.alarm_messages[wakeup_time].append(signal)
 
+    def list_alarms(self, receiver=None):
+        """ returns alarms set for uuid
+        :param: uuid - agent uuid
+        :returns: list of tuples (time, message)
+        """
+        if receiver is None:
+            return [(t, m) for t, m in self.alarm_messages.items()]
+        L = []
+        for timestamp, messages in self.alarm_messages.items():
+            L.extend([(timestamp, m) for m in messages if m.receiver == receiver])
+        return L
+
     def clear_alarms(self, uuid=None, message=None):
         """
         :param uuid: setter of the alarm.
         :param message: alarm message to be removed.
         """
         if message is not None:  # the long way.
-            for time in self.alarm_messages.copy():
-                self.alarm_messages[time] = [s for s in self.alarm_messages[time] if s != message]
+            for timestamp in self.alarm_messages.copy():
+                self.alarm_messages[timestamp] = [s for s in self.alarm_messages[timestamp] if s != message]
         elif uuid is not None:
-            for time in self.alarm_messages.copy():
-                self.alarm_messages[time] = [s for s in self.alarm_messages[time] if s.receiver != uuid]
+            for timestamp in self.alarm_messages.copy():
+                self.alarm_messages[timestamp] = [s for s in self.alarm_messages[timestamp] if s.receiver != uuid]
         else:
             self.alarms.clear()
             self.alarm_messages.clear()
@@ -600,38 +627,35 @@ class Scheduler(object):
         distributes the mail, so that when the scheduler pauses, new users
         can debug the agents starting with their fresh state with new messages.
         """
-        while self.mail_queue:
-            msg = self.mail_queue.popleft()
-            if isinstance(msg, AgentMessage):
-                topic = msg.topic
-                receiver = msg.receiver
-                recipients = set()
-                # 1. collect the list of recipients
+        for msg in self.mail_queue:
+            assert isinstance(msg, AgentMessage)
+            topic = msg.topic
+            receiver = msg.receiver
+            recipients = set()
+            # 1. collect the list of recipients
 
-                if (None, receiver) in self.mailing_lists:  # then it's a tracked receiver.
-                    recipients.update(self.mailing_lists[(None, receiver)])
+            if (None, receiver) in self.mailing_lists:  # then it's a tracked receiver.
+                recipients.update(self.mailing_lists[(None, receiver)])
 
-                if (topic, None) in self.mailing_lists:  # then it's a tracked topic
-                    recipients.update(self.mailing_lists[(topic, None)])
+            if (topic, None) in self.mailing_lists:  # then it's a tracked topic
+                recipients.update(self.mailing_lists[(topic, None)])
 
-                if (topic, receiver) in self.mailing_lists:  # then it's tracked topic and receiver.
-                    recipients.update(self.mailing_lists[(topic, receiver)])
+            if (topic, receiver) in self.mailing_lists:  # then it's tracked topic and receiver.
+                recipients.update(self.mailing_lists[(topic, receiver)])
 
-                # 2. distribute the mail.
-                if recipients:  # receiver is not in self.mailing_lists, so s/h/it might be moving.
-                    self.send_to_recipients(msg=msg, recipients=recipients)
-                else:
-                    self.log(level=DEBUG, msg="{} is not registered on a mailing list.".format(receiver))
+            # 2. distribute the mail.
+            if recipients:  # receiver is not in self.mailing_lists, so s/h/it might be moving.
+                self.send_to_recipients(msg=msg, recipients=recipients)
             else:
-                self.log(level=WARNING,
-                         msg="Discovered a non-AgentMessage in the inbox. The message is dumped.")
+                self.log(level=DEBUG, msg="{} is not registered on a mailing list.".format(receiver))
+        self.mail_queue.clear()
 
     def send_to_recipients(self, msg, recipients):
         """ Distributes AgentMessages to all registered recipients.
         :param msg: an instance of AgentMessage
         :param recipients: The registered recipients
         """
-        for uuid in recipients:  # this loop is necessary as a tracker may be on the reciever.
+        for uuid in recipients:  # this loop is necessary as a tracker may be on the receiver.
             agent = self.agents.get(uuid, None)
             if agent is None:
                 continue
@@ -639,15 +663,7 @@ class Scheduler(object):
             if msg.receiver == uuid:
                 agent.inbox.append(msg)  # original message
             else:
-                try:
-                    msg_copy = msg.copy()
-                except Exception as e:
-                    m = """
-                    Message could not be copied:\n\t{}\ncausing exception:\n{}. 
-                    Please check that the contents in the message is decoupled from other objects.
-                    """.format(str(msg), str(e))
-                    raise SchedulerException(m)
-
+                msg_copy = msg.copy()
                 agent.inbox.append(msg_copy)
 
     def pause(self):
@@ -670,8 +686,10 @@ class Scheduler(object):
             raise ValueError("Alarm time is in the past")
 
         self.clock.set_alarm(delay=delay, signal=alarm_message, ignore_alarm_if_idle=ignore_alarm_if_idle)
-        # if ignore_alarm_if_idle is False:
-        #     self._must_run_until_alarm_expires = True
+
+    def list_alarms(self, receiver=None):
+        """returns list of alarms for uuid"""
+        return self.clock.list_alarms(receiver)
 
     def subscribe(self, subscriber, target=None, topic=None):
         """ subscribe lets the Agent react to SubscribeMessage and adds the subscriber.
